@@ -21,7 +21,7 @@ use LaTeXML::Util::Pathname;
 use base qw(LaTeXML::Common::Object);
 
 #**********************************************************************
-my $LTX_NAMESPACE = "http://dlmf.nist.gov/LaTeXML";    # [CONSTANT]
+our $LTX_NAMESPACE = "http://dlmf.nist.gov/LaTeXML";    # [CONSTANT]
 
 sub new {
   my ($class, %options) = @_;
@@ -31,8 +31,8 @@ sub new {
     namespace_errors        => 0,
     %options }, $class;
   $$self{xpath}->registerFunction('match-font', \&LaTeXML::Common::Font::match_font);
-  $self->registerNamespace('xml', "http://www.w3.org/XML/1998/namespace");
-  $self->registerDocumentNamespace('xml', "http://www.w3.org/XML/1998/namespace");
+  $self->registerNamespace('xml', $LaTeXML::Common::XML::XML_NS);
+  $self->registerDocumentNamespace('xml', $LaTeXML::Common::XML::XML_NS);
   return $self; }
 
 sub setDocType {
@@ -59,6 +59,7 @@ sub loadSchema {
     $self->registerNamespace(xlink => "http://www.w3.org/1999/xlink");         # Needed for SVG
     $self->registerNamespace(m     => "http://www.w3.org/1998/Math/MathML");
     $self->registerNamespace(xhtml => "http://www.w3.org/1999/xhtml");
+
     $$self{permissive} = 1; }    # Actually, they could have declared all sorts of Tags....
 
   my ($type, @data) = @{ $$self{schemadata} };
@@ -99,6 +100,7 @@ sub compileSchema {
     foreach my $classname (sort keys %$defs) {
       print $classname. ':=(' . join(',', sort keys %{ $$self{schemaclass}{$classname} }) . ')' . "\n"; } }
   foreach my $tag (sort keys %{ $$self{tagprop} }) {
+    next if $tag =~ /^!/;    # Ignore top-level negated (only make sense in content model)
     print $tag
       . '{' . join(',', sort keys %{ $$self{tagprop}{$tag}{attributes} }) . '}'
       . '(' . join(',', sort keys %{ $$self{tagprop}{$tag}{model} }) . ')' . "\n"; }
@@ -106,7 +108,7 @@ sub compileSchema {
 
 sub loadCompiledSchema {
   my ($self, $file) = @_;
-  NoteBegin("Loading compiled schema $file");
+  ProgressSpinup("Loading compiled schema $file");
   my $MODEL;
   open($MODEL, '<', $file) or Fatal('I/O', $file, undef, "Cannot open Compiled Model $file for reading", $!);
   my $line;
@@ -126,7 +128,7 @@ sub loadCompiledSchema {
       Fatal('internal', $file, undef, "Compiled model '$file' is malformatted at \"$line\""); }
   }
   close($MODEL);
-  NoteEnd("Loading compiled schema $file");
+  ProgressSpindown("Loading compiled schema $file");
   return; }
 
 #**********************************************************************
@@ -185,7 +187,7 @@ sub getNamespace {
       $ns = "http://example.com/namespace" . (++$$self{namespace_errors}));
     Error('malformed', $codeprefix, undef,
       "No namespace has been registered for prefix '$codeprefix' (in code)",
-      "Using '$ns' isntead"); }
+      "Using '$ns' instead"); }
   return $ns; }
 
 sub registerDocumentNamespace {
@@ -209,7 +211,8 @@ sub getDocumentNamespacePrefix {
    # Get the prefix associated with the namespace url, noting that for elements, it might by "#default",
    # but for attributes would never be.
     my $docprefix = (!$forattribute && $$self{document_namespace_prefixes}{ "DEFAULT#" . $namespace })
-      || $$self{document_namespace_prefixes}{$namespace};
+      || $$self{document_namespace_prefixes}{$namespace}
+      || (($namespace ne $LTX_NAMESPACE) && $$self{code_namespace_prefixes}{$namespace});
     if ((!defined $docprefix) && !$probe) {
       $self->registerDocumentNamespace($docprefix = "namespace" . (++$$self{namespace_errors}), $namespace);
       Warn('malformed', $namespace, undef,
@@ -237,7 +240,7 @@ sub decodeQName {
   my ($self, $codetag) = @_;
   if ($codetag =~ /^([^:]+):(.+)$/) {
     my ($prefix, $localname) = ($1, $2);
-    return (undef, $codetag) if $prefix eq 'xml';
+    return (undef,                        $codetag) if $prefix eq 'xml';
     return ($self->getNamespace($prefix), $localname); }
   else {
     return (undef, $codetag); } }
@@ -252,7 +255,7 @@ sub encodeQName {
 # NOTE: Reconsider how _Capture_ & _WildCard_ should be integrated!?!
 sub getNodeQName {
   my ($self, $node) = @_;
-  my $type = $node->nodeType;
+  my $type = ($node ? $node->nodeType : -1);
   if ($type == XML_TEXT_NODE) {
     return '#PCDATA'; }
   elsif ($type == XML_DOCUMENT_NODE) {
@@ -264,7 +267,7 @@ sub getNodeQName {
   elsif ($type == XML_DTD_NODE) {
     return '#DTD'; }
   elsif ($type == XML_NAMESPACE_DECL) {
-    my $ns = $node->declaredURI;
+    my $ns     = $node->declaredURI;
     my $prefix = $ns && $self->getNamespacePrefix($ns, 0, 1);
     return ($prefix ? 'xmlns:' . $prefix : 'xmlns'); }
   # Need others?
@@ -273,7 +276,7 @@ sub getNodeQName {
       "Should not ask for Qualified Name for node of type $type: " . Stringify($node));
     return; }
   else {
-    my $ns = $node->namespaceURI;
+    my $ns     = $node->namespaceURI;
     my $prefix = $ns && $self->getNamespacePrefix($ns, 0, 1);
     return ($prefix ? $prefix . ":" . $node->localname : $node->localname); } }
 
@@ -292,7 +295,7 @@ sub getNodeDocumentQName {
   elsif ($type == XML_DTD_NODE) {
     return '#DTD'; }
   elsif ($type == XML_NAMESPACE_DECL) {
-    my $ns = $node->declaredURI;
+    my $ns     = $node->declaredURI;
     my $prefix = $ns && $self->getDocumentNamespacePrefix($ns, 0, 1);
     return ($prefix ? 'xmlns:' . $prefix : 'xmlns'); }
   # Need others?
@@ -301,7 +304,7 @@ sub getNodeDocumentQName {
       "Should not ask for Qualified Name for node of type $type: " . Stringify($node));
     return; }
   else {
-    my $ns = $node->namespaceURI;
+    my $ns     = $node->namespaceURI;
     my $prefix = $ns && $self->getDocumentNamespacePrefix($ns, 0, 1);
     return ($prefix ? $prefix . ":" . $node->localname : $node->localname); } }
 
@@ -311,8 +314,8 @@ sub getNodeDocumentQName {
 # into a prefixed name using the Code's prefixes
 # NOTE: Used only for DTD
 sub recodeDocumentQName {
-  my ($self, $docQName) = @_;
-  my ($docprefix, $name) = (undef, $docQName);
+  my ($self,      $docQName) = @_;
+  my ($docprefix, $name)     = (undef, $docQName);
   if ($docQName =~ /^(#PCDATA|#Comment|ANY|#ProcessingInstruction|#Document)$/) {
     return $docQName; }
   else {
@@ -371,9 +374,8 @@ sub canContain {
   my ($self, $tag, $childtag) = @_;
   $self->loadSchema unless $$self{schema_loaded};
   # Handle obvious cases explicitly.
-  return 0 if $tag eq '#PCDATA';
-  return 0 if $tag eq '#Comment';
-  return 1 if $tag =~ /(.*?:)?_Capture_$/;             # with or without namespace prefix
+  return 0 if !$tag || ($tag eq '#PCDATA') || ($tag eq '#Comment');
+  return 1 if $tag =~ /(.*?:)?_Capture_$/;                          # with or without namespace prefix
   return 1 if $tag eq '_WildCard_';
   return 1 if $childtag =~ /(.*?:)?_Capture_$/;
   return 1 if $childtag eq '_WildCard_';
@@ -382,10 +384,28 @@ sub canContain {
   return 1 if $childtag eq '#DTD';
   #  return 1 if $$self{permissive}; # No DTD? Punt!
   return 1 if $$self{permissive} && ($tag eq '#Document') && ($childtag ne '#PCDATA'); # No DTD? Punt!
-         # Else query tag properties.
+      # Else query tag properties.
   my $model = $$self{tagprop}{$tag}{model};
-  return $$model{ANY} || $$model{$childtag}; }
+  if (!$model && ($tag =~ /^(\w*):/)) {
+    my $xtag = $1 . ':*';
+    $model = $$self{tagprop}{$xtag}{model}; }
+  my ($chns, $chname) = ($childtag =~ /^([^:]*):(.*)$/ ? ($1, $2) : (undef, $childtag));
+  # NOTE: Maybe recode this to (carefully) use 0 & 1, and defined, //
+  if ($chns) {
+    return ($$model{$childtag} ? 1
+      : ($$model{"!$childtag"} ? 0
+        : ($$model{"$chns:*"} ? 1
+          : ($$model{"!$chns:*"} ? 0
+            : ($$model{'*:*'} ? 1
+              : 0))))); }
+  else {
+    return ($$model{$childtag} ? 1
+      : ($$model{"!$childtag"} ? 0
+        : ($$model{'*:*'} ? 1
+          : 0))); } }
 
+# NOTE: Currently the Document class already allows ANY namespaced attributes!
+# (which is very unmodular, although it does have to arrange for namespace declarations)
 sub canHaveAttribute {
   my ($self, $tag, $attrib) = @_;
   $self->loadSchema unless $$self{schema_loaded};
@@ -396,7 +416,29 @@ sub canHaveAttribute {
   return 0 if $tag eq '#DTD';
   return 1 if $tag =~ /(.*?:)?_Capture_$/;
   return 1 if $$self{permissive};
-  return $$self{tagprop}{$tag}{attributes}{$attrib}; }
+  my ($tagns,  $tagname)  = ($tag    =~ /^(\w+):(\w*)$/ ? ($1, $2) : ('', $tag));
+  my ($attrns, $attrname) = ($attrib =~ /^(\w+):(\w*)$/ ? ($1, $2) : ('', $attrib));
+  my $attr = $$self{tagprop}{$tag}{attributes};
+  # Check:
+  #    data for tag:  tagns:tagname, tagns:* (or even *:tagname ???)
+  if (!$attr && ($tag =~ /^(\w*):/)) {
+    my $xtag = $1 . ':*';
+    $attr = $$self{tagprop}{$xtag}{attributes}; }
+  # Test for allowance OR exclusion of an attribute, possibly w/namespace
+  # testing the most specific form first
+  # NOTE: We could be turning !ns:name into 0 and be checking for defined ???
+  if ($attrns) {
+    return ($$attr{$attrib} ? 1
+      : ($$attr{"!$attrib"} ? 0
+        : ($$attr{"$attrns:*"} ? 1
+          : ($$attr{"!$attrns:*"} ? 0
+            : ($$attr{'*:*'} ? 1
+              : ($$attr{'!*:*'} ? 0
+                : 0)))))); }
+  else {
+    return ($$attr{$attrib} ? 1
+      : ($$attr{"!$attrib"} ? 0
+        : 0)); } }
 
 sub isInSchemaClass {
   my ($self, $classname, $tag) = @_;
@@ -404,16 +446,20 @@ sub isInSchemaClass {
   my $class = $$self{schemaclass}{$classname};
   return $class && $$class{$tag}; }
 
+sub isKnownTag {
+  my ($self, $name) = @_;
+  return exists($$self{tagprop}{$name}); }
+
 #**********************************************************************
 sub describeModel {
   my ($self) = @_;
-  print STDERR "Doctype\n";
+  Debug("Doctype");
   foreach my $tag (sort keys %{ $$self{tagprop} }) {
     if (my $model = $$self{tagprop}{$tag}{model}) {
       if (keys %$model) {
-        print STDERR "$tag can contain " . join(', ', sort keys %{ $$self{tagprop}{$tag}{model} }) . "\n"; } }
+        Debug("$tag can contain " . join(', ', sort keys %{ $$self{tagprop}{$tag}{model} })); } }
     else {
-      print STDERR "$tag is empty\n"; }
+      Debug("$tag is empty"); }
   }
   return; }
 
@@ -422,7 +468,7 @@ sub describeModel {
 
 __END__
 
-=pod 
+=pod
 
 =head1 NAME
 
@@ -481,7 +527,7 @@ that are relevant to L<LaTeXML>.
 The `code' mapping is the one used in code implementing packages, and in
 particular, constructors defined within those packages.  The prefix C<ltx>
 is used consistently to refer to L<LaTeXML>'s own namespace
-(C<http://dlmf.nist.gov/LaTeXML)>. 
+(C<http://dlmf.nist.gov/LaTeXML)>.
 
 The other mapping, the `document' mapping, is used in the created document;
 this may be different from the `code' mapping in order to accommodate
@@ -514,7 +560,7 @@ Return the namespace url for the given C<$prefix>.
 
 =item C<< $boole = $model->canContain($tag,$childtag); >>
 
-Returns whether an element with qualified name C<$tag> can contain an element 
+Returns whether an element with qualified name C<$tag> can contain an element
 with qualified name C<$childtag>.
 The tag names #PCDATA, #Document, #Comment and #ProcessingInstruction
 are specially recognized.
