@@ -37,7 +37,7 @@ my $XMLPARSER = LaTeXML::Common::XML::Parser->new();    # [CONSTANT]
 sub new {
   my ($class, $model, $name) = @_;
   my $self = { name => $name, model => $model,
-    modules => [], elementdefs => {}, defs => {}, elements => {},
+    modules           => [], elementdefs => {}, defs => {}, elements => {},
     internal_grammars => 0 };
   weaken($$self{model});    # circular back ref; weaked so can be garbage collected.
   bless $self, $class;
@@ -50,28 +50,28 @@ sub addSchemaDeclaration {
 
 sub loadSchema {
   my ($self) = @_;
-  NoteBegin("Loading RelaxNG $$self{name}");
+  ProgressSpinup("Loading RelaxNG $$self{name}");
   # Scan the schema file(s), and extract the info
   my @schema = $self->scanExternal($$self{name});
-  if ($LaTeXML::Common::Model::RelaxNG::DEBUG) {
-    print STDERR "========================\nRaw Schema\n";
+  if ($LaTeXML::DEBUG{relaxng}) {
+    Debug("========================\nRaw Schema");
     map { showSchema($_) } @schema; }
   @schema = map { $self->simplify($_) } @schema;
-  if ($LaTeXML::Common::Model::RelaxNG::DEBUG) {
-    print STDERR "========================\nSimplified Schema\n";
+  if ($LaTeXML::DEBUG{relaxng}) {
+    Debug("========================\nSimplified Schema");
     map { showSchema($_) } @schema;
-    print STDERR "========================\nElements\n";
+    Debug("========================\nElements");
     foreach my $tag (sort keys %{ $$self{elements} }) {
       showSchema(['element', $tag, @{ $$self{elements}{$tag} }]); }
-    print STDERR "========================\nModules\n";
+    Debug("========================\nModules");
     foreach my $mod (@{ $$self{modules} }) {
       showSchema($mod); } }
 
   # The resulting @schema should contain the "start" of the grammar.
   my ($startcontent) = $self->extractContent('#Document', @schema);
   $$self{model}->addTagContent('#Document', sort keys %$startcontent);
-  if ($LaTeXML::Common::Model::RelaxNG::DEBUG) {
-    print STDERR "========================\nStart\n" . join(', ', sort keys %$startcontent) . "\n"; }
+  if ($LaTeXML::DEBUG{relaxng}) {
+    Debug("========================\nStart\n" . join(', ', sort keys %$startcontent)); }
 
   # NOTE: Do something automatic about this too!?!
   # We'll need to generate namespace prefixes for all namespaces found in the doc!
@@ -79,9 +79,9 @@ sub loadSchema {
 
   # Distill the info into allowed children & attributes for each element.
   foreach my $tag (sort keys %{ $$self{elements} }) {
-    if ($tag eq 'ANY') {
+    if ($tag eq '*:*') {
       # Ignore any internal structure (side effect of restricted names)
-      $$self{model}->addTagContent($tag, 'ANY');
+      $$self{model}->addTagContent($tag, '*:*');
       next; }
     my @body = @{ $$self{elements}{$tag} };
     my ($content, $attributes) = $self->extractContent($tag, @body);
@@ -93,7 +93,7 @@ sub loadSchema {
       my $name = $1;
       my ($content, $attributes) = $self->extractContent($symbol, $$self{defs}{$symbol});
       $$self{model}->setSchemaClass($name, $content); } }
-  NoteEnd("Loading RelaxNG $$self{name}");
+  ProgressSpindown("Loading RelaxNG $$self{name}");
   return; }
 
 # Return two hashrefs for content & attributes
@@ -105,9 +105,9 @@ sub extractContent {
     my $item = shift(@body);
     if (ref $item eq 'ARRAY') {
       my ($op, $name, @args) = @$item;
-      if    ($op eq 'attribute')  { $attr{$name}  = 1; }
-      elsif ($op eq 'elementref') { $child{$name} = 1; }
-      elsif ($op eq 'doc')        { }
+      if    ($op eq 'attribute')   { $attr{$name}  = 1; }
+      elsif ($op eq 'elementref')  { $child{$name} = 1; }
+      elsif ($op eq 'doc')         { }
       elsif ($op eq 'combination') { push(@body, @args); }
       elsif ($op eq 'grammar')     { push(@body, $self->extractStart(@args)); }
       elsif ($op eq 'module')      { push(@body, $self->extractStart(@args)); }
@@ -122,7 +122,8 @@ sub extractContent {
         $child{$name} = 1; }    # ???
       elsif (($op eq 'value') || ($op eq 'data')) {
         $child{'#PCDATA'} = 1; }
-      else { print STDERR "Unknown child $op [$name] of element $tag in extractContent\n"; } }
+      else { Warn('unexpected', $name, undef,
+          "Unknown child $op [$name] of element $tag in extractContent\n"); } }
     elsif ($item eq '#PCDATA') { $child{'#PCDATA'} = 1; } }
   return ({%child}, {%attr}); }
 
@@ -181,10 +182,10 @@ local @LaTeXML::Common::Model::RelaxNG::PATHS = ();
 sub scanExternal {
   my ($self, $name, $inherit_ns) = @_;
   my $modname = $name; $modname =~ s/\.rn(g|c)$//;
-  my $paths = [@LaTeXML::Common::Model::RelaxNG::PATHS, @{ $STATE->lookupValue('SEARCHPATHS') }];
+  my $paths   = [@LaTeXML::Common::Model::RelaxNG::PATHS, @{ $STATE->lookupValue('SEARCHPATHS') }];
   if (my $schemadoc = LaTeXML::Common::XML::RelaxNG->new($name, searchpaths => $paths)) {
     my $uri = $schemadoc->URI;
-    NoteBegin("Loading RelaxNG schema from $uri");
+    ProgressSpinup("Loading RelaxNG schema from $uri");
     local @LaTeXML::Common::Model::RelaxNG::PATHS
       = (pathname_directory($schemadoc->URI), @LaTeXML::Common::Model::RelaxNG::PATHS);
     my $node = $schemadoc->documentElement;
@@ -194,7 +195,7 @@ sub scanExternal {
       next if $nsuri =~ m|^http://relaxng.org|;    # Ignore RelaxNG namespaces(!!)
       $$self{model}->registerDocumentNamespace($prefix, $nsuri); }
     my $mod = (['module', $modname, $self->scanPattern($node, $inherit_ns)]);
-    NoteEnd("Loading RelaxNG schema from $uri");
+    ProgressSpindown("Loading RelaxNG schema from $uri");
     return $mod; }
   else {
     Warn('expected', $name, undef, "Failed to find RelaxNG schema for name '$name'");
@@ -203,7 +204,7 @@ sub scanExternal {
 sub getRelaxOp {
   my ($node) = @_;
   return unless $node->nodeType == XML_ELEMENT_NODE;
-  my $ns = $node->namespaceURI;
+  my $ns     = $node->namespaceURI;
   my $prefix = $ns && $RNGNSMAP{$ns};
   return ($prefix ? $prefix : "{$ns}") . ":" . $node->localname; }
 
@@ -266,19 +267,19 @@ sub scanPattern_element {
         $self->scanChildren($ns, @children)]); }
   else {
     my $namenode = shift(@children);
-    my @names = $self->scanNameClass($namenode, $ns);
+    my @names    = $self->scanNameClass($namenode, 0, $ns);
     return map { ['element', $_, $self->scanChildren($ns, @children)] } @names; } }
 
 sub scanPattern_attribute {
   my ($self, $ns, $node) = @_;
-  $ns = $node->getAttribute('ns');    # ONLY explicit declaration!
+  my $xns      = $node->getAttribute('ns');    # explicit declaration!
   my @children = getElements($node);
   if (my $name = $node->getAttribute('name')) {
-    return (['attribute', $$self{model}->encodeQName($ns, $name),
+    return (['attribute', $$self{model}->encodeQName($xns, $name),
         $self->scanChildren($ns, @children)]); }
   else {
     my $namenode = shift(@children);
-    my @names = $self->scanNameClass($namenode, $ns);
+    my @names    = $self->scanNameClass($namenode, 1, $ns);
     return map { ['attribute', $_, $self->scanChildren($ns, @children)] } @names; } }
 
 sub scanChildren {
@@ -293,18 +294,18 @@ sub scanGrammarItem {
   my ($self, $node, $inherit_ns) = @_;
   if (my $relaxop = getRelaxOp($node)) {
     my @children = getElements($node);
-    my $ns       = $node->getAttribute('ns') || $inherit_ns;   # Possibly bind new namespace
-                                                               # The start element's content is returned
+    my $ns       = $node->getAttribute('ns') || $inherit_ns;    # Possibly bind new namespace
+        # The start element's content is returned
     if ($relaxop eq 'rng:start') {
       return (['start', undef, $self->scanChildren($ns, @children)]); }
     elsif ($relaxop eq 'rng:define') {
       my $name = $node->getAttribute('name');
-      my $op = $node->getAttribute('combine') || '';
+      my $op   = $node->getAttribute('combine') || '';
       return (['def' . $op, $name, $self->scanChildren($ns, @children)]); }
     elsif ($relaxop eq 'rng:div') {
       return $self->scanGrammarContent($ns, @children); }
     elsif ($relaxop eq 'rng:include') {
-      my $name = $node->getAttribute('href');
+      my $name  = $node->getAttribute('href');
       my $paths = [@LaTeXML::Common::Model::RelaxNG::PATHS, @{ $STATE->lookupValue('SEARCHPATHS') }];
       if (my $schemadoc = LaTeXML::Common::XML::RelaxNG->new($name, searchpaths => $paths)) {
         local @LaTeXML::Common::Model::RelaxNG::PATHS
@@ -332,30 +333,36 @@ sub scanGrammarItem {
     return (); } }
 
 sub scanNameClass {
-  my ($self, $node, $ns) = @_;
+  my ($self, $node, $forattr, $ns) = @_;
   my $relaxop = getRelaxOp($node);
   if ($relaxop eq 'rng:name') {
-    return ($$self{model}->encodeQName($ns, $node->textContent)); }
+    my ($decns, $name) = $$self{model}->decodeQName($node->textContent);
+    $ns = $decns if $decns;
+    return ($$self{model}->encodeQName(($forattr ? undef : $ns), $name)); }
   elsif ($relaxop eq 'rng:anyName') {
-    Info('unexpected', $relaxop, undef, "Can't handle RelaxNG operation '$relaxop'",
-      "Treating " . ToString($node) . " as ANY")
-      if $node->hasChildNodes;
-    return ('ANY'); }
+    my @exceptions = ();    # Check for exceptions!
+    if (my @children = getElements($node)) {
+      @exceptions = map { $self->scanNameClass($_, $forattr, $ns) } @children; }
+    return ('*:*', @exceptions); }
   elsif ($relaxop eq 'rng:nsName') {
-    Info('unexpected', $relaxop, undef, "Can't handle RelaxNG operation '$relaxop'",
-      "Treating " . ToString($node) . " as ANY");
-    # NOTE: We _could_ conceivably use a namespace predicate,
-    # but Model has to be extended to support it!
-    return ('ANY'); }
+    my @exceptions = ();    # Check for exceptions!
+    if (my @children = getElements($node)) {
+      @exceptions = map { $self->scanNameClass($_, $forattr, $ns) } @children; }
+    return ($$self{model}->encodeQName($node->getAttribute('ns') || $ns, '*'), @exceptions); }
   elsif ($relaxop eq 'rng:choice') {
     my %names = ();
     foreach my $choice ($node->childNodes) {
-      map { $names{$_} = 1 } $self->scanNameClass($choice, $ns); }
-    return ($names{ANY} ? ('ANY') : sort keys %names); }
+      map { $names{$_} = 1 } $self->scanNameClass($choice, $forattr, $ns); }
+    return (sort keys %names); }
+  elsif ($relaxop eq 'rng:except') {
+    my %names = ();
+    foreach my $choice (getElements($node)) {
+      map { $names{$_} = 1 } $self->scanNameClass($choice, $forattr, $ns); }
+    return map { "!$_" } (sort keys %names); }
   else {
     my $op = $node->nodeName;
     Fatal('misdefined', $op, undef,
-      "Expected a RelaxNG name element (rng:name|rng:anyName|rng:nsName|rng:choice), got '$op'");
+"Expected a RelaxNG name element (rng:name|rng:anyName|rng:nsName|rng:choice|rng:except), got '$op'");
     return; } }
 
 #======================================================================
@@ -383,7 +390,7 @@ sub extractStart {
       if    ($op eq 'start')   { push(@starts, @args); }
       elsif ($op eq 'module')  { push(@starts, $self->extractStart(@args)); }
       elsif ($op eq 'grammar') { push(@starts, $self->extractStart(@args)); }
-    } }
+  } }
   return @starts; }
 
 # NOTE: Reconsider this process.
@@ -426,7 +433,7 @@ sub simplify {
     elsif ($op eq 'module') {
       my $module = ['module', $name];
       push(@{ $$self{modules} }, $module);    # Keep in order: push first, then scan contents
-      push(@$module, $self->simplify_args($binding, $parentbinding, $container, @args));
+      push(@$module,             $self->simplify_args($binding, $parentbinding, $container, @args));
       return ($module); }
     elsif ($op eq 'element') {
       @args = $self->simplify_args($binding, $parentbinding, "element:$name", @args);
@@ -439,7 +446,7 @@ sub simplify {
       return (['ref', $qname]); }
     elsif ($op =~ /^def(choice|interleave|)$/) {
       my $combination = $1 || 'group';
-      my $qname = $binding . ":" . $name;
+      my $qname       = $binding . ":" . $name;
       $$self{usesname}{$qname}{$container} = 1 if $container;
       @args = $self->simplify_args($binding, $parentbinding, "pattern:$qname", @args);
 
@@ -506,12 +513,12 @@ sub showSchema {
   if (ref $item eq 'ARRAY') {
     my ($op, $name, @args) = @$item;
     if ($op eq 'doc') { $name = "..."; @args = (); }
-    print STDERR "" . (' ' x (2 * $level)) . $op . ($name ? " " . $name : '') . "\n";
+    Debug("" . (' ' x (2 * $level)) . $op . ($name ? " " . $name : ''));
 
     foreach my $arg (@args) {
       showSchema($arg, $level + 1); } }
   else {
-    print STDERR "" . (' ' x (2 * $level)) . $item . "\n"; }
+    Debug("" . (' ' x (2 * $level)) . $item); }
   return; }
 
 #======================================================================
@@ -519,7 +526,8 @@ sub showSchema {
 #======================================================================
 # The svg schema can only just barely be read in and recognized,
 # but it is structured in a way that makes a joke of our attempt at automatic documentation
-my $SKIP_SVG = 1;    # [CONFIGURABLE?]
+my $SKIP_SVG  = 1;    # [CONFIGURABLE?]
+my $SKIP_ARIA = 1;
 
 sub documentModules {
   my ($self) = @_;
@@ -527,8 +535,8 @@ sub documentModules {
   $$self{defined_patterns} = {};
   foreach my $module (@{ $$self{modules} }) {
     my ($op, $name, @content) = @$module;
-    next if $SKIP_SVG && $name =~ /:svg:/;    # !!!!
-    $name =~ s/^urn:x-LaTeXML:RelaxNG://;     # Remove the urn part.
+    next if $SKIP_SVG && $name =~ /:svg:/;                        # !!!!
+    $name =~ s/^urn:x-LaTeXML:RelaxNG://;                         # Remove the urn part.
     $docs = join("\n", $docs,
       "\\begin{schemamodule}{$name}",
       (map { $self->toTeX($_) } @content),
@@ -544,6 +552,7 @@ sub cleanTeX {
   $string =~ s/\#/\\#/g;
   $string =~ s/<([^>]*)>/\\texttt{$1}/g;    # An apparent convention <sometext> == ttfont?
   $string =~ s/_/\\_/g;
+  $string =~ s/\*/any/g;                    # ????
   return $string; }
 
 sub cleanTeXName {
@@ -580,7 +589,6 @@ sub toTeX {
       my $content = join(' ', map { $self->toTeX($_) } @spec);
       return "\\item[\\textit{Start}]\\textbf{==}\\ $content" . ($docs ? " \\par$docs" : ''); }
     elsif ($op eq 'grammar') {    # Don't otherwise mention it?
-##      join("\n",'\item[\textit{Grammar}:] '.$name,
       return join("\n", map { $self->toTeX($_) } @data); }
     elsif ($op eq 'module') {
       $name =~ s/^urn:x-LaTeXML:RelaxNG://;    # Remove the urn part.
@@ -606,7 +614,8 @@ sub toTeX_ref {
 sub toTeX_def {
   my ($self, $combiner, $name, @data) = @_;
   my $qname = $name;
-  $name =~ s/^\w+://;      # Strip off qualifier!!!! (watch for clash in docs?)
+  return "" if ($name =~ /\baria\b/) && $SKIP_ARIA;
+  $name =~ s/^\w+://;    # Strip off qualifier!!!! (watch for clash in docs?)
   $name = cleanTeX($name);
   my ($docs, @spec)    = $self->toTeXExtractDocs(@data);
   my ($attr, $content) = $self->toTeXBody(@spec);
@@ -617,8 +626,8 @@ sub toTeX_def {
     return "\\patternadd{$name}{$docs}{$body}\n"; }
   #      elsif((scalar(@data)==1) && (ref $data[0] eq 'ARRAY') && ($data[0][0] eq 'grammar')){
   else {
-    $attr = '\item[\textit{Attributes:}] \textit{empty}' if !$attr && ($name =~ /\\_attributes/);
-    $content = '\textit{empty}' if !$content && ($name =~ /\\_model/);
+    $attr    = '\item[\textit{Attributes:}] \textit{empty}' if !$attr    && ($name =~ /\\_attributes/);
+    $content = '\textit{empty}'                             if !$content && ($name =~ /\\_model/);
     my $body = $attr;
     $body .= '\item[\textit{Content}:] ' . $content if $content;
     my ($xattr, $xcontent) = $self->toTeXBody($$self{defs}{$qname});
