@@ -22,7 +22,7 @@ use base qw(LaTeXML::Post::Processor);
 
 #======================================================================
 # Options:
-#   dpi : dots per inch for target medium.
+#   DPI : dots per inch for target medium.
 #   ignore_options  : list of graphicx options to be ignored.
 #   warn_options    : list of graphicx options to cause warning if used.
 #   trivial_scaling : If true, web images that only need scaling will be used as-is
@@ -48,20 +48,20 @@ use base qw(LaTeXML::Post::Processor);
 sub new {
   my ($class, %options) = @_;
   my $self = $class->SUPER::new(%options);
-  $$self{dppt} = (($options{dpi} || 90) / 72.0);    # Dots per point.
+  $$self{DPI}             = $options{DPI};
   $$self{ignore_options}  = $options{ignore_options}  || [];
   $$self{trivial_scaling} = $options{trivial_scaling} || 1;
   $$self{graphics_types}  = $options{graphics_types}
     || [qw(svg png gif jpg jpeg
-      eps ps ai pdf)];
+      eps ps postscript ai pdf)];
   $$self{type_properties} = $options{type_properties}
     || {
     ai => { destination_type => 'png',
       transparent => 1,
-      prescale => 1, ncolors => '400%', quality => 90, unit => 'point' },
+      prescale    => 1, ncolors => '400%', quality => 90, unit => 'point' },
     pdf => { destination_type => 'png',
       transparent => 1,
-      prescale => 1, ncolors => '400%', quality => 90, unit => 'point' },
+      prescale    => 1, ncolors => '400%', quality => 90, unit => 'point' },
     ps => { destination_type => 'png', transparent => 1,
       prescale => 1, ncolors => '400%', quality => 90, unit => 'point' },
     eps => { destination_type => 'png', transparent => 1,
@@ -89,13 +89,28 @@ sub process {
   my ($self, $doc, @nodes) = @_;
   local $LaTeXML::Post::Graphics::SEARCHPATHS
     = [map { pathname_canonical($_) } $self->findGraphicsPaths($doc), $doc->getSearchPaths];
-  NoteProgressDetailed(" [Using graphicspaths: "
-      . join(', ', @$LaTeXML::Post::Graphics::SEARCHPATHS) . "]");
+  Debug(" [Using graphicspaths: "
+      . join(', ', @$LaTeXML::Post::Graphics::SEARCHPATHS) . "]") if $LaTeXML::DEBUG{images};
   foreach my $node (@nodes) {
     $self->processGraphic($doc, $node); }
   $doc->closeCache;    # If opened.
   return $doc; }
 
+# Need a proper API to query PI's, probably in Post
+# But Core isn't respecting @at@document@begin from preloaded packages,
+# so here we're testing BOTH a straight DPI PI and the options from latexml.sty (ugh)
+sub getParameter {
+  my ($self, $doc, $parameter) = @_;
+  my $value;
+  foreach my $pi (@{ $$doc{processingInstructions} }) {
+    if ($pi =~ /^\s*$parameter\s*=\s*([\"\'])(.*?)\1\s*$/) {
+      $value = $2; }
+    elsif ($pi =~ /^\s*package\s*=\s*([\"\'])latexml\1\s*options\s*=\s*([\"\'])(.*?)\2\s*$/i) {
+      my $options = $3;
+      if ($options =~ /\b$parameter\s*=\s*(\d+)\b/i) {
+        $value = $1; } } }
+  return $value || $$self{$parameter}; }
+
 #======================================================================
 # Potentially customizable operations.
 # find graphics file
@@ -113,22 +128,30 @@ sub findGraphicsPaths {
 
 sub getGraphicsSourceTypes {
   my ($self) = @_;
-  return @{ $$self{graphics_types} }; }
+  return @{ $$self{graphics_types} }, map { uc($_); } @{ $$self{graphics_types} }; }
 
 # Return the pathname to an appropriate image.
 sub findGraphicFile {
   my ($self, $doc, $node) = @_;
-  if (my $name = $node->getAttribute('graphic')) {
+  if (my $source = $node->getAttribute('graphic')) {
     # Find all acceptable image files, in order of search paths
-    my @paths = pathname_findall($name, paths => $LaTeXML::Post::Graphics::SEARCHPATHS,
+    my ($dir, $name, $reqtype) = pathname_split($source);
+    # Ignore the requested type? Or should it increase desirability?
+    #
+    # If this is *NOT* a known graphics type, it would be best to treat it as
+    # a name suffix (e.g. name.1 from name.1.png)
+    if ((length($reqtype) > 0) && !(grep { $_ eq lc($reqtype) } $$self{graphics_types})) {
+      $name .= ".$reqtype"; }
+    my $file  = pathname_concat($dir, $name);
+    my @paths = pathname_findall($file, paths => $LaTeXML::Post::Graphics::SEARCHPATHS,
       # accept empty type, incase bad type name, but actual file's content is known type.
       types => ['', $self->getGraphicsSourceTypes]);
     my ($best, $bestpath) = (-1, undef);
     # Now, find the first image that is either the correct type,
     # or has the most desirable type mapping
     foreach my $path (@paths) {
-      my $type  = pathname_type($path);
-      my $props = $$self{type_properties}{$type};
+      my $type         = pathname_type($path);
+      my $props        = $$self{type_properties}{$type};
       my $desirability = $$props{desirability} || ($type eq ($$props{destination_type} || 'notype') ? 10 : 0);
       if ($desirability > $best) {
         $best     = $desirability;
@@ -149,7 +172,7 @@ sub getTransform {
 sub getTypeProperties {
   my ($self, $source, $options) = @_;
   my ($dir,  $name,   $ext)     = pathname_split($source);
-  my $props = $$self{type_properties}{$ext};
+  my $props = $$self{type_properties}{ lc($ext) };
   if (!$props) {
     # If we don't have a known file type, try a bit harder (maybe less efficient)
     if (my $type = image_type($source)) {
@@ -160,16 +183,36 @@ sub getTypeProperties {
 # width and height.
 sub setGraphicSrc {
   my ($self, $node, $src, $width, $height) = @_;
-  $node->setAttribute('imagesrc',    $src);
-  $node->setAttribute('imagewidth',  $width) if defined $width;
-  $node->setAttribute('imageheight', $height) if defined $height;
+  # If we are on windows, the $src path will be used for a URI context from the 'imagesrc' attribute,
+  # so we can already switch it to the canonical slashified form
+  $node->setAttribute('imagesrc', pathname_to_url($src));
+  if (defined $width) {
+    # final formats mostly expect numeric literals to be integers
+    $width = int($width) if ($width =~ /^\d*\.\d*$/);
+    $node->setAttribute('imagewidth', $width); }
+  if (defined $height) {
+    # final formats mostly expect numeric literals to be integers
+    $height = int($height) if ($height =~ /^\d*\.\d*$/);
+    $node->setAttribute('imageheight', $height); }
+  if ($width and $height) {
+    my $aspect_class = "ltx_img_square";
+    if ($width > 1.24 * $height) {
+      $aspect_class = "ltx_img_landscape"; }
+    elsif ($height > 1.24 * $width) {
+      $aspect_class = "ltx_img_portrait" }
+    my $class = $node->getAttribute('class');
+    $class = ($class ? $class . ' ' : '') . $aspect_class;
+    $node->setAttribute('class', $class); }
   return; }
 
 sub processGraphic {
   my ($self, $doc, $node) = @_;
   my $source = $self->findGraphicFile($doc, $node);
   if (!$source) {
-    Warn('expected', 'source', $node, "No graphic source specified; skipping"); return; }
+    Warn('expected', 'source', $node, "No graphic source found; skipping",
+      "source was " . ($node->getAttribute('graphic') || 'none'));
+    $node->setAttribute('imagesrc', $node->getAttribute('graphic'));
+    return; }
   my $transform = $self->getTransform($node);
   my ($image, $width, $height) = $self->transformGraphic($doc, $node, $source, $transform);
   # $image should probably be relative already, except corner applications?
@@ -182,18 +225,23 @@ sub processGraphic {
 
 sub transformGraphic {
   my ($self, $doc, $node, $source, $transform) = @_;
+  my @parameters = (
+    DPI      => $self->getParameter($doc, 'DPI') || $LaTeXML::Util::Image::DPI,
+    magnify  => $self->getParameter($doc, 'magnify'),
+    upsample => $self->getParameter($doc, 'upsample'),
+    zoomout  => $self->getParameter($doc, 'zoomout'));
   my $sourcedir = $doc->getSourceDirectory;
   ($sourcedir) = $doc->getSearchPaths unless $sourcedir;    # Fishing...
   my ($reldir, $name, $srctype)
     = pathname_split(pathname_relative($source, $sourcedir));
-  my $key = (ref $self) . ':' . join('|', "$reldir$name.$srctype",
-    map { join(' ', @$_) } @$transform);
-  NoteProgressDetailed("\n[Processing $source as key=$key]");
-
   my %properties = $self->getTypeProperties($source, $transform);
   return Warn('unexpected', 'graphics_format', undef,
     "Don't know what to do with graphics file format '$source'") unless %properties;
   my $type = $properties{destination_type} || $srctype;
+  my $key  = (ref $self) . ':' . join('|', "$reldir$name.$srctype.$type",
+    map { join(' ', @$_) } @$transform);
+  Debug("Processing $source as key=$key") if $LaTeXML::DEBUG{images};
+
   my $dest = $self->desiredResourcePathname($doc, $node, $source, $type);
   if (my $prev = $doc->cacheLookup($key)) {                 # Image was processed on previous run?
     if ($prev =~ /^(.*?)\|(\d*)\|(\d*)$/) {
@@ -204,7 +252,8 @@ sub transformGraphic {
       if ((!defined $dest) || ($cached eq $dest)) {
         my $absdest = pathname_absolute($cached, $doc->getDestinationDirectory);
         if (pathname_timestamp($source) <= pathname_timestamp($absdest)) {
-          NoteProgressDetailed(" [Reuse $cached @ " . ($width || '?') . " x " . ($height || '?') . "]");
+          Debug(" [Reuse $cached @ " . ($width || '?') . " x " . ($height || '?') . "]")
+            if $LaTeXML::DEBUG{images};
           return ($cached, $width, $height); } } } }
   # Trivial scaling case: Use original image with (at most) different width & height.
   my $triv_scaling = $$self{trivial_scaling} && ($type eq $srctype)
@@ -243,11 +292,11 @@ sub transformGraphic {
     # Now IFF that is a valid relative path WITHIN the site directory, we'll use it.
     # Otherwise, we'd better fall back to a generated name.
     if (!pathname_is_contained($absdest, $doc->getSiteDirectory)) {
-      $dest = $self->generateResourcePathname($doc, $node, $source, $type);
+      $dest    = $self->generateResourcePathname($doc, $node, $source, $type);
       $absdest = $doc->checkDestination($dest); }
 
-    NoteProgressDetailed(" [Destination $absdest]");
-    ($width, $height) = image_graphicx_trivial($source, $transform, ddpt => $$self{ddpt});
+    Debug(" [Destination $absdest]") if $LaTeXML::DEBUG{images};
+    ($width, $height) = image_graphicx_trivial($source, $transform, @parameters);
     if (!($width && $height)) {
       if (!image_can_image()) {
         Warn('imageprocessing', 'imagesize', undef,
@@ -259,23 +308,24 @@ sub transformGraphic {
           "Couldn't get usable image size for $source"); } }
     pathname_copy($source, $absdest)
       or Warn('I/O', $absdest, undef, "Couldn't copy $source to $absdest", "Response was: $!");
-    NoteProgressDetailed(" [Copied to $dest @ " . ($width || '?') . " x " . ($height || '?') . "]"); }
+    Debug(" [Copied to $dest @ " . ($width || '?') . " x " . ($height || '?') . "]")
+      if $LaTeXML::DEBUG{images}; }
   else {
     # With a complex transformation, we really needs a new name (well, don't we?)
     $dest = $self->generateResourcePathname($doc, $node, $source, $type) unless $dest;
     my $absdest = $doc->checkDestination($dest);
-    NoteProgressDetailed(" [Destination $absdest]");
+    Debug(" [Destination $absdest]") if $LaTeXML::DEBUG{images};
     ($image, $width, $height) = image_graphicx_complex($source, $transform,
-      ddpt => $$self{ddpt}, background => $$self{background}, %properties);
+      @parameters, background => $$self{background}, %properties);
     if (!($image && $width && $height)) {
       Warn('expected', 'image', undef,
         "Couldn't get usable image for $source");
       return; }
-    NoteProgressDetailed(" [Writing to $absdest]");
+    Debug(" [Writing to $absdest]") if $LaTeXML::DEBUG{images};
     image_write($image, $absdest) or return; }
 
   $doc->cacheStore($key, "$dest|" . ($width || '') . '|' . ($height || ''));
-  NoteProgressDetailed(" [done with $key]");
+  Debug(" [done with $key]") if $LaTeXML::DEBUG{images};
   return ($dest, $width, $height); }
 
 #======================================================================
