@@ -40,44 +40,43 @@ sub new {
 
 sub toProcess {
   my ($self, $doc) = @_;
-  return $doc->findnode('//ltx:bibliography'); }
+  return $doc->findnodes('//ltx:bibliography'); }
 
+# Whooo, this is turning into a confusing API!
+# Potentially multiple biblist's in each document
+# potentially each one split into multiple output documents (need that option!)
+# Inevitably duplicates?
 sub process {
-  my ($self, $doc, $bib) = @_;
+  my ($self, $doc, @bibs) = @_;
+  my @docs = ($doc);
+  foreach my $bib (@bibs) {
+    next if $doc->findnodes('.//ltx:bibitem', $bib);    # Already populated?
 
-  my @bib = ($doc);
-  return $doc if $doc->findnodes('//ltx:bibitem', $bib);    # Already populated?
+    local $LaTeXML::Post::MakeBibliography::NUMBER = 0;
+    local %LaTeXML::Post::MakeBibliography::STYLE =
+      (map { ($_ => $bib->getAttribute($_)) } qw(bibstyle citestyle sort));
 
-  local $LaTeXML::Post::MakeBibliography::NUMBER = 0;
-  local %LaTeXML::Post::MakeBibliography::STYLE =
-    (map { ($_ => $bib->getAttribute($_)) } qw(bibstyle citestyle sort));
+    my $entries = $self->getBibEntries($doc, $bib);
+    # Remove any bibentry's (these should have been converted to bibitems)
+    $doc->removeNodes($doc->findnodes('//ltx:bibentry'));
+    foreach my $biblist ($doc->findnodes('//ltx:biblist')) {
+      $doc->removeNodes($biblist) unless element_nodes($biblist); }
 
-  my $entries = $self->getBibEntries($doc);
-  # Remove any bibentry's (these should have been converted to bibitems)
-  $doc->removeNodes($doc->findnodes('//ltx:bibentry'));
-  foreach my $biblist ($doc->findnodes('//ltx:biblist')) {
-    $doc->removeNodes($biblist) unless element_nodes($biblist); }
-
-  if ($$self{split}) {
-    # Separate by initial.
-    my $split = {};
-    foreach my $sortkey (keys %$entries) {
-      my $entry = $$entries{$sortkey};
-      $$split{ $$entry{initial} }{$sortkey} = $entry; }
-    @bib = map { $self->rescan($_) }
-      $self->makeSubCollectionDocuments($doc, $bib,
-      map { ($_ => $self->makeBibliographyList($doc, $_, $$split{$_})) }
-        sort keys %$split); }
-  else {
-    $doc->addNodes($bib, $self->makeBibliographyList($doc, undef, $entries));
-    @bib = ($self->rescan($doc)); }
-  return @bib; }
-
-# Try to preserve the original form & case of the provided Bibkeys
-# HOWEVER, we downcase them before indexing & looking them up!!!!
-sub normalizeBibKey {
-  my ($key) = @_;
-  return lc($key); }
+    if ($$self{split}) {
+      # Separate by initial.
+      my $split = {};
+      foreach my $sortkey (keys %$entries) {
+        my $entry = $$entries{$sortkey};
+        $$split{ $$entry{initial} }{$sortkey} = $entry; }
+      @docs = map { $self->rescan($_) }
+        $self->makeSubCollectionDocuments($doc, $bib,
+        map { ($_ => $self->makeBibliographyList($doc, $bib, $_, $$split{$_})) }
+          sort keys %$split); }
+    else {
+      $doc->addNodes($bib, $self->makeBibliographyList($doc, $bib, undef, $entries));
+      #      @docs = ($self->rescan($doc)); }
+      $self->rescan($doc); } }
+  return @docs; }
 
 # ================================================================================
 # Bibliographies can be specified either
@@ -93,16 +92,22 @@ sub normalizeBibKey {
 # In principle, we ought to look for named files also in the file cache (eg. from {filecontents})
 sub getBibliographies {
   my ($self, $doc) = @_;
-  my @bibnames = ();
-  # use the commandline bibliographies, if explicitly given.
+  my @bibnames         = ();
+  my $fromBibliography = 0;    # coming from an 'ltx:bibliography'
+                               # use the commandline bibliographies, if explicitly given.
   if ($$self{bibliographies} && scalar(@{ $$self{bibliographies} })) {
     @bibnames = @{ $$self{bibliographies} }; }
   else {
     my $bibnode = $doc->findnode('//ltx:bibliography');
-    if (my $files = $bibnode->getAttribute('files')) {
-      @bibnames = map { $_ . '.bib' } split(',', $files); } }
-  my @paths = $doc->getSearchPaths;
-  my @bibs  = ();
+    if (my $files = $bibnode->getAttribute('files')
+      || $bibnode->parentNode->getAttribute('files')    # !!!!!
+    ) {
+      $fromBibliography = 1;
+      @bibnames         = split(',', $files); } }
+  my @paths   = $doc->getSearchPaths;
+  my @bibs    = ();
+  my @rawbibs = ();
+ # Collect the "ready" bibliographies, while accumulating all raw sources for a single conversion pass
   foreach my $bib (@bibnames) {
     my $ref = ref $bib;
     my $bibdoc;
@@ -114,20 +119,21 @@ sub getBibliographies {
       Error('unexpected', $ref, $self,
         "Don't know how to convert '$bib' (a '$ref') into a Bibliography document"); }
     elsif (pathname_is_literaldata($bib)) {
-      # Assume (for now) that the data is BibTeX text;
-      # [another possibility is serialized xml!]
-      $bibdoc = $self->convertBibliography($doc, $bib); }
-    elsif ($bib =~ /\.xml/) {
+      push(@rawbibs, $bib);
+      next; }
+    elsif ($bib =~ /\.xml$/) {
       $bibdoc = $doc->newFromFile($bib); }    # doc will do the searching...
-    elsif ($bib =~ /\.bib(?:\.xml)?$/) {
-      my $name = $1;
+    elsif ($bib =~ /\.bib(?:\.xml)?$/ || $fromBibliography) {
       # NOTE: We should also use kpsewhich to get the effects of $BIBINPUTS?
       # NOTE: When better integrated with Core, should also check for cached bib documents.
-      if (my $xmlpath = pathname_find($bib, paths => [@paths], types => ['xml'])) {
+      my $xmlbib = $bib;
+      $xmlbib .= '.bib' if $fromBibliography && !($xmlbib =~ /\.bib$/);
+      if (my $xmlpath = pathname_find($xmlbib, paths => [@paths], types => ['xml'])) {
         $bibdoc = $doc->newFromFile($xmlpath); }    # doc will do the searching...
       elsif (my $bibpath = pathname_find($bib, paths => [@paths], types => ['bib'])
         || pathname_kpsewhich($bib)) {
-        $bibdoc = $self->convertBibliography($doc, $bibpath); }
+        push(@rawbibs, $bibpath);
+        next; }
       else {
         Error('missing_file', $bib, $self,
           "Couldn't find Bibliography '$bib'",
@@ -137,7 +143,27 @@ sub getBibliographies {
     else {
       Info('expected', $bib, $self,
         "Couldn't find usable Bibliography for '$bib'"); } }
-  NoteProgress(" [using bibliographies "
+  # Lastly, If we found any raw .bib files/literaldata, convert and include them.
+  if (@rawbibs) {
+    my $raw;
+    if (scalar(@rawbibs) == 1) {    # Single, just convert as-is
+      $raw = $rawbibs[0]; }
+    else {
+      $raw = 'literal:';
+      for my $rawbib (@rawbibs) {    # Multiple, arrange into a single conversion payload
+        if ($rawbib =~ s/literal\://) {
+          $raw .= $rawbib; }
+        else {
+          # TODO: Is this a memory concern for large bib files?
+          if (open(my $bibfh, '<', $rawbib)) {
+            $raw .= join("", <$bibfh>);
+            close $bibfh; }
+          else {
+            Info("open", $rawbib, $self, "Couldn't open file $rawbib"); } }
+        $raw .= "%\n"; } }
+    my $bibdoc = $self->convertBibliography($doc, $raw);
+    push(@bibs, $bibdoc) if $bibdoc; }
+  NoteLog("MakeBibliography: using bibliographies "
       . join(',', map { (length($_) > 100 ? substr($_, 100) . '...' : $_) } @bibnames)
       . "]");
   return @bibs; }
@@ -155,105 +181,126 @@ sub convertBibliography {
   my ($self, $doc, $bib) = @_;
   require LaTeXML;
   require LaTeXML::Common::Config;
-  my @packages =
-    my @preload = ();
-  # Might want/need to preload more (all?) packages, but at least do inputenc!
-  foreach my $po ($self->find_documentclass_and_packages($doc)) {
+  my @preload = ();    # custom macros often used in e.g. howpublished field
+                       # need to preload all packages used by the main article
+  my ($classdata, @packages)     = $self->find_documentclass_and_packages($doc);
+  my ($class,     $classoptions) = @$classdata;
+  if ($class) {
+
+    if ($classoptions) {
+      push(@preload, "[$classoptions]$class.cls"); }
+    else {
+      push(@preload, "$class.cls"); } }
+  foreach my $po (@packages) {
     my ($pkg, $options) = @$po;
-    if ($pkg eq 'inputenc') {
-      push(@preload, "[$options]$pkg"); } }
-  NoteProgress(" [Converting bibliography $bib ...");
+    if ($options) {
+      push(@preload, "[$options]$pkg.sty"); }
+    else {
+      push(@preload, "$pkg.sty"); } }
+  my $bibname = pathname_is_literaldata($bib) ? 'Anonymous Bib String' : $bib;
+  my $stage   = "Recursive MakeBibliography $bibname";
+  ProgressSpinup($stage);
   my $bib_config = LaTeXML::Common::Config->new(
+    recursive      => 1,
     cache_key      => 'BibTeX',
     type           => "BibTeX",
     post           => 0,
     format         => 'dom',
     whatsin        => 'document',
     whatsout       => 'document',
+    includestyles  => 1,
     bibliographies => [],
+    paths          => $$doc{searchpaths},
     (@preload ? (preload => [@preload]) : ()));
   my $bib_converter = LaTeXML->get_converter($bib_config);
   # Tricky and HACKY, we need to release the log to capture the inner workings separately.
   # ->bind_log analog:
   my $biblog = '';
-  my $biblog_handle;
-  open($biblog_handle, ">>", \$biblog) or Error("Can't redirect STDERR to log for inner bibliography converter!");
-  *BIB_STDERR_SAVED = *STDERR;
-  *STDERR           = *$biblog_handle;
+###  my $biblog_handle;
+###  open($biblog_handle, ">>", \$biblog) or Error("Can't redirect STDERR to log for inner bibliography converter!");
+###  *BIB_STDERR_SAVED = *STDERR;
+###  *STDERR           = *$biblog_handle;
   # end ->bind_log
 
   $bib_converter->prepare_session($bib_config);
   my $response = $bib_converter->convert($bib);
 
   # ->flush_log analog:
-  close $biblog_handle;
-  *STDERR = *BIB_STDERR_SAVED;
+###  close $biblog_handle;
+###  *STDERR = *BIB_STDERR_SAVED;
   # end ->flush_log
 
   # Trim log to look internal and report.
-  $biblog =~ s/^.+?\(Digesting/\n\(Digesting/s;
-  $biblog =~ s/Conversion complete:.+$//s;
-  print STDERR $biblog;
+###  $biblog =~ s/^.+?\(Digesting/\n\(Digesting/s;
+###  $biblog =~ s/Conversion complete:.+$//s;
+###  print STDERR $biblog;
   MergeStatus($$bib_converter{latexml}{state});
 
-  # TODO: We need to handle the logging properly, it's a bit of a mess for nested ->convert() calls
+  ProgressSpindown($stage);
   if (my $bibdoc = $$response{result}) {
-    NoteProgress("... converted!]");
     return $doc->new($bibdoc, sourceDirectory => '.'); }
-  else {
-    NoteProgress("... Failed!]");
-    return; } }
+  return; }
 
 # ================================================================================
 # Get all cited bibentries from the requested bibliography files.
-# Sort (cited) bibentries on author+year+title, [NOT on the key!!!]
+# Sort (cited) bibentries on author+year+title, [NOT on the key, or even proper BibTeX choices!!!]
 # and then check whether author+year is unique!!!
 # Returns a list of hashes containing:
 #  bibkey : the bibliographic entry's key
+#  citedkey : the key as cited (can be different in case)
 #  bibentry : the bibentry node
 #  citations : array of bib keys that are cited somewhere within this bibentry
 #  referrers : array of ID's of places that refer to this bibentry
 #  suffix    : a,b... if adjacent author/year are identical.
-
+# NOTE: biblist's now have @lists to restrict to include ONLY
+# those bibitems which have been referenced by bibref's for one of those lists!
+# [If a bibref gives several lists, and entries for those lists end up in several bibliographies,
+# first 'list' determines where the idref points]
 sub getBibEntries {
-  my ($self, $doc) = @_;
+  my ($self, $doc, $bib) = @_;
 
-  # First, scan the bib files for all ltx:bibentry's, (hash key is bibkey)
+  # First, scan the bib files for all ltx:bibentry's,
+  # %entries is  a hash with the key in lowercase!
   # Also, record the citations from each bibentry to others.
   my %entries = ();
   foreach my $bibdoc ($self->getBibliographies($doc)) {
+    my @lists = split(/\s+/, $doc->findnode('//ltx:bibliography')->getAttribute('lists') || 'bibliography');
     foreach my $bibentry ($bibdoc->findnodes('//ltx:bibentry')) {
-      my $bibkey = normalizeBibKey($bibentry->getAttribute('key'));
-      my $bibid  = $bibentry->getAttribute('xml:id');
-      $entries{$bibkey}{bibkey}   = $bibkey;
-      $entries{$bibkey}{bibentry} = $bibentry;
-      $entries{$bibkey}{citations} = [map { normalizeBibKey($_) } grep { $_ } map { split(',', $_->value) }
-          $bibdoc->findnodes('.//@bibrefs', $bibentry)];
-      # Also, register the key with the DB, if the bibliography hasn't already been scanned.
-      $$self{db}->register("BIBLABEL:$bibkey", id => $bibid);
-    } }
+      my $bibkey   = $bibentry->getAttribute('key');
+      my $lcbibkey = lc($bibkey);
+      my $bibid    = $bibentry->getAttribute('xml:id');
+      $entries{$lcbibkey}{bibkey}    = $bibkey;
+      $entries{$lcbibkey}{bibentry}  = $bibentry;
+      $entries{$lcbibkey}{citations} = [grep { $_ } map { split(',', $_->value) }
+          $bibdoc->findnodes('.//@bibrefs', $bibentry)]; } }
   # Now, collect all bibkeys that were cited in other documents (NOT the bibliography)
   # And note any referrers to them (also only those outside the bib)
-  my $citestar = $$self{db}->lookup('BIBLABEL:*');
-
-  my @queue = ();
+  my @lists    = split(/\s+/, $bib->getAttribute('lists') || 'bibliography');
+  my $citestar = grep { $$self{db}->lookup("BIBLABEL:$_:*"); } @lists;
+  my @queue    = ();
   foreach my $dbkey ($$self{db}->getKeys) {
-    if ($dbkey =~ /^BIBLABEL:(.*)$/) {
-      my $bibkey = $1;
-      if (my $referrers = $$self{db}->lookup($dbkey)->getValue('referrers')) {
+    if ($dbkey =~ /^BIBLABEL:(.*?):(.*)$/) {
+      my ($list, $bibkey) = ($1, $2);
+      next unless grep { $_ eq $list; } @lists;
+      my $lcbibkey   = lc($bibkey);
+      my $bibdbentry = $$self{db}->lookup($dbkey);
+      if (my $referrers = $bibdbentry->getValue('referrers')) {
         foreach my $refr (keys %$referrers) {
           my ($rid, $e, $t) = ($refr, undef, undef);
           while ($rid && ($e = $$self{db}->lookup("ID:$rid")) && (($t = ($e->getValue('type') || '')) ne 'ltx:bibitem')) {
             $rid = $e->getValue('parent'); }
-          if(! $e){
-            Warn('expected','entry',undef,
-                 "Didn't find an entry for reference id=$rid"); }
+          if (!$e) {
+            Warn('expected', 'entry', undef,
+              "Didn't find an entry for reference id=$rid"); }
           elsif ($t ne 'ltx:bibitem') {
-            $entries{$bibkey}{referrers}{$refr} = 1; } }
-        push(@queue, $bibkey) if keys %{ $entries{$bibkey}{referrers} }; }
-      elsif ($citestar) {    # If \cite{*} include all of them.
+            if (my $prevkey = $entries{$lcbibkey}{citedkey}) {
+              Warn('unexpected', 'bibkey', undef,
+                "Case mismatch in bib key '$prevkey' vs '$bibkey'") if $prevkey ne $bibkey; }
+            $entries{$lcbibkey}{citedkey} = $bibkey;            # Store bibkey with case as CITED
+            $entries{$lcbibkey}{referrers}{$refr} = 1; } }
+        push(@queue, $bibkey) if keys %{ $entries{$lcbibkey}{referrers} }; }
+      elsif ($citestar) {                                       # If \cite{*} include all of them.
         push(@queue, $bibkey); } } }
-
   # For each bibkey in the queue, complete and include the entry
   # And add any keys cited from within each include entry
   my %seen_keys    = ();
@@ -263,8 +310,9 @@ sub getBibEntries {
     next if $seen_keys{$bibkey};    # Done already.
     $seen_keys{$bibkey} = 1;
     next if $bibkey eq '*';
-    if (my $bibentry = $entries{$bibkey}{bibentry}) {
-      my $entry = $entries{$bibkey};
+    my $lcbibkey = lc($bibkey);
+    if (my $bibentry = $entries{$lcbibkey}{bibentry}) {
+      my $entry = $entries{$lcbibkey};
       # Extract names, year and title from bibentry.
       my $names     = '';
       my $sortnames = '';
@@ -275,16 +323,17 @@ sub getBibEntries {
       elsif (scalar(@names)) {
         $sortnames = join(' ', map { getNameText($doc, $_) } @names);
         my @ns = map { $_ && $_->textContent } map { $doc->findnodes('ltx:surname', $_) } @names;
-        if    (@ns > 2) { $names = $ns[0] . ' et al'; }
+        if (@ns > 2)    { $names = $ns[0] . ' et al'; }
         elsif (@ns > 1) { $names = $ns[0] . ' and ' . $ns[1]; }
-        else { $names = $ns[0]; } }
+        else            { $names = $ns[0]; } }
       elsif (my $t = $doc->findnode('ltx:bib-title', $bibentry)) {
         $sortnames = $names = $t->textContent; }
-      my $date = $doc->findnode('ltx:bib-date[@role="publication"] | ltx:bib-type', $bibentry);
-      my $title = $doc->findnode('ltx:bib-title', $bibentry);
-      $date  = ($date  ? $date->textContent  : '');
-      $title = ($title ? $title->textContent : '');
-      $$entry{ay} = "$names.$date";
+      my $date  = $doc->findnode('ltx:bib-date[@role="publication"] | ltx:bib-type', $bibentry);
+      my $title = $doc->findnode('ltx:bib-title',                                    $bibentry);
+      $date            = ($date ? $date->textContent : '');
+      $date            = $1 if $date && $date =~ /^(\d\d\d\d)/;
+      $title           = ($title ? $title->textContent : '');
+      $$entry{ay}      = "$names.$date";
       $$entry{initial} = $doc->initial($names, 1);
       # Include this entry keyed using a sortkey.
       $$included{ lc(join('.', $sortnames, $date, $title, $bibkey)) } = $entry;
@@ -296,9 +345,9 @@ sub getBibEntries {
   foreach my $sortkey (keys %$included) {
     my $entry  = $$included{$sortkey};
     my $bibkey = $$entry{bibkey};
-    map { $entries{ normalizeBibKey($_) }{bibreferrers}{$bibkey} = 1 } @{ $$entry{citations} }; }
+    map { $entries{ lc($_) }{bibreferrers}{$bibkey} = 1 } @{ $$entry{citations} }; }
 
-  NoteProgress(" [" . (scalar keys %entries) . " bibentries, " . (scalar keys %$included) . " cited]");
+  NoteLog("MakeBibliography: " . (scalar keys %entries) . " bibentries, " . (scalar keys %$included) . " cited");
   Warn('expected', 'bibkeys', undef,
     "Missing bibkeys " . join(', ', sort keys %missing_keys)) if keys %missing_keys;
 
@@ -340,22 +389,31 @@ sub getNameText {
 # Convert hash of bibentry(s) into biblist of bibitem(s)
 
 sub makeBibliographyList {
-  my ($self, $doc, $initial, $entries) = @_;
-  my $id = $doc->getDocumentElement->getAttribute('xml:id') || 'bib';
+  my ($self, $doc, $bib, $initial, $entries) = @_;
+  my $id = $bib->getAttribute('xml:id')
+    || $doc->getDocumentElement->getAttribute('xml:id') || 'bib';
   $id .= ".L1";
   $id .= ".$initial" if $initial;
   return ['ltx:biblist', { 'xml:id' => $id },
-    map { $self->formatBibEntry($doc, $$entries{$_}) } $doc->unisort(keys %$entries)]; }
+    map { $self->formatBibEntry($doc, $bib, $$entries{$_}) } $doc->unisort(keys %$entries)]; }
 
 # ================================================================================
+# NOTE: With multiple bibliographies, the ID of the bibentry isn't necessarily
+# the ID in the "local" bibliography! (ie. bibentry can be repeated in the document!)
 sub formatBibEntry {
-  my ($self, $doc, $entry) = @_;
+  my ($self, $doc, $bib, $entry) = @_;
   my $bibentry   = $$entry{bibentry};
+  my $citedkey   = $$entry{citedkey} || $$entry{bibkey};
   my $id         = $bibentry->getAttribute('xml:id');
   my $key        = $bibentry->getAttribute('key');
   my $type       = $bibentry->getAttribute('type');
   my @blockspecs = @{ $FMT_SPEC{$type} || [] };
 
+  # Patch the entry's id in case there are multiple bibs.
+  if (my $bibid = $bib->getAttribute('xml:id')
+    || $doc->getDocumentElement->getAttribute('xml:id') || 'bib') {
+    $id =~ s/^bib//; $id = $bibid . $id; }
+
   local $LaTeXML::Post::MakeBibliography::SUFFIX = $$entry{suffix};
   my $number = ++$LaTeXML::Post::MakeBibliography::NUMBER;
 
@@ -370,6 +428,9 @@ sub formatBibEntry {
   # Set up authors and fullauthors tags
   my @names = $doc->findnodes('ltx:bib-name[@role="author"]/ltx:surname', $bibentry);
   @names = $doc->findnodes('ltx:bib-name[@role="editor"]/ltx:surname', $bibentry) unless @names;
+  my $etal = 0;
+  if (@names && ($names[-1]->toString eq 'others')) {                                    # Magic!
+    $etal = 1; }
   if (@names > 2) {
     push(@tags, ['ltx:tag', { role => 'authors', class => 'ltx_bib_author' },
         $doc->cloneNodes($names[0]->childNodes),
@@ -436,8 +497,8 @@ sub formatBibEntry {
         $aa = substr($aa, 0, 3) . "+"; } }
     else {
       $aa = uc(substr($rfnames[0]->textContent, 0, 3)); }
-    my $yrtext = (@year ? join('', map { (ref $_ ? $_->textContent : $_); } @year) : '');
-    my $yy = (length($yrtext) >= 2 ? substr($yrtext, 2, 2) : $yrtext);
+    my $yrtext = (@year                ? join('', map { (ref $_ ? $_->textContent : $_); } @year) : '');
+    my $yy     = (length($yrtext) >= 2 ? substr($yrtext, 2, 2) : $yrtext);
     push(@tags, ['ltx:tag', { role => 'refnum', class => 'ltx_bib_abbrv', open => '[', close => ']' },
         $aa . $yy . ($$entry{suffix} || '')]); }
 
@@ -463,12 +524,12 @@ sub formatBibEntry {
     foreach my $row (@$blockspec) {
       my ($xpath, $punct, $pre, $class, $op, $post) = @$row;
       my $negated = $xpath =~ s/^!\s*//;
-      my @nodes = ($xpath eq 'true' ? () : $doc->findnodes($xpath, $bibentry));
+      my @nodes   = ($xpath eq 'true' ? () : $doc->findnodes($xpath, $bibentry));
       next if ($xpath ne 'true') && ($negated ? @nodes : !@nodes);
       push(@x, $punct) if $punct && @x;
-      push(@x, $pre) if $pre;
+      push(@x, $pre)                                                                           if $pre;
       push(@x, ['ltx:text', { class => 'ltx_bib_' . $class }, &$op($doc->cloneNodes(@nodes))]) if $op;
-      push(@x, $post) if $post; }
+      push(@x, $post)                                                                          if $post; }
     push(@blocks, ['ltx:bibblock', { 'xml:space' => 'preserve' }, @x]) if @x;
   }
   # Add a Cited by block.
@@ -480,7 +541,7 @@ sub formatBibEntry {
   push(@blocks, ['ltx:bibblock', { class => 'ltx_bib_cited' },
       "Cited by: ", $doc->conjoin(",\n", @citedby), '.']) if @citedby;
 
-  return ['ltx:bibitem', { 'xml:id' => $id, key => $key, type => $type, class => "ltx_bib_$type" },
+  return ['ltx:bibitem', { 'xml:id' => $id, key => $citedkey, type => $type, class => "ltx_bib_$type" },
     (@tags ? (['ltx:tags', {}, @tags]) : ()),
     @blocks]; }
 
@@ -506,10 +567,20 @@ sub do_name {
 
 sub do_names {
   my (@names) = @_;
-  my @stuff = ();
+  my @stuff   = ();
+  my $sep     = (scalar(@names) > 2 ? ', ' : ' ');
+  my $etal    = 0;
+  if (@names && ($names[-1]->textContent eq 'others')) {    # Magic!
+    pop(@names);
+    $etal = 1; }
+  my $n = scalar(@names);
   while (my $name = shift(@names)) {
-    push(@stuff, (@names ? ', ' : ' and ')) if @stuff;
+    if (@stuff) {
+      push(@stuff, $sep);
+      push(@stuff, 'and ') if !$etal && !@names; }
     push(@stuff, do_name($name)); }
+  if ($etal) {
+    push(@stuff, $sep, ['ltx:text', { class => 'ltx_bib_etal' }, 'et al.']); }
   return @stuff; }
 
 sub do_names_short {
@@ -570,7 +641,7 @@ sub do_crossref {
   return (['ltx:cite', {},
       ['ltx:bibref', { bibrefs => $node->getAttribute('bibrefs'), show => 'title, author' }]]); }
 
-my $LINKS =                                       # CONSTANT
+my $LINKS =    # CONSTANT
   "ltx:bib-links | ltx:bib-review | ltx:bib-identifier | ltx:bib-url";
 
 sub do_links {
@@ -709,11 +780,11 @@ sub do_links {
     @META_BLOCK],
   website =>
     [[['ltx:bib-name[@role="author"]', '', '', 'author', \&do_authors, ''],
-      ['ltx:bib-name[@role="editor"]',      '', '', 'editor', \&do_editorsA, ''],
-      ['ltx:bib-date[@role="publication"]', '', '', 'year',   \&do_year,     ''],
-      ['ltx:title',                         '', '', 'title',  \&do_any,      ''],
-      ['ltx:bib-type',                      '', '', 'type',   \&do_any,      ''],
-      ['! ltx:bib-type', '', '', 'type', sub { ('(Website)'); }, '']],
+      ['ltx:bib-name[@role="editor"]',      '', '', 'editor', \&do_editorsA,          ''],
+      ['ltx:bib-date[@role="publication"]', '', '', 'year',   \&do_year,              ''],
+      ['ltx:title',                         '', '', 'title',  \&do_any,               ''],
+      ['ltx:bib-type',                      '', '', 'type',   \&do_any,               ''],
+      ['! ltx:bib-type',                    '', '', 'type',   sub { ('(Website)'); }, '']],
     [['ltx:bib-organization', ', ', ' ', 'publisher', \&do_any, ''],
       ['ltx:bib-place', ', ', '', 'place', \&do_any, ''],
       ['true', '.']],
@@ -745,4 +816,3 @@ $FMT_SPEC{techreport}            = $FMT_SPEC{report};
 
 # ================================================================================
 1;
-
