@@ -15,6 +15,7 @@ use strict;
 use warnings;
 use LaTeXML::Post;
 use LaTeXML::Util::Pathname;
+use LaTeXML::Common::Error;
 use base qw(LaTeXML::Post::MathProcessor LaTeXML::Post::LaTeXImages);
 
 sub new {
@@ -58,6 +59,10 @@ sub convertNode {
   my ($self, $doc, $xmath, $style) = @_;
   my $math = $xmath->parentNode;
   my $tex  = $self->extractTeX($doc, $math);
+  if (!$tex) {
+    Warn('imageprocessing', 'TeX', undef,
+      "No TeX string for math node");
+    return (); }
   my $type = $$self{imagetype};
   #  next if !(defined $tex) || ($tex =~ /^\s*$/);
   my $key = (ref $self) . ':' . $type . ':' . $tex;
@@ -68,7 +73,7 @@ sub convertNode {
     return { processor => $self, mimetype => $MIMETYPES{$type},
       src => $reldest, width => $width, height => $height, depth => $depth }; }
   else {
-    print STDERR "Couldn't find image for '$key'\n";
+    Warn('expected', 'image', $doc, "Couldn't find image for '$key'");
     return {}; } }
 
 # Definitions needed for processing inline & display math images
