@@ -38,9 +38,10 @@ our @EXPORT = qw( &pathname_find &pathname_findall &pathname_kpsewhich
   &pathname_split &pathname_directory &pathname_name &pathname_type
   &pathname_timestamp
   &pathname_concat
-  &pathname_relative &pathname_absolute
+  &pathname_relative &pathname_absolute &pathname_to_url
   &pathname_is_absolute &pathname_is_contained
   &pathname_is_url &pathname_is_literaldata
+  &pathname_is_raw &pathname_is_reloadable
   &pathname_protocol
   &pathname_cwd &pathname_chdir &pathname_mkdir &pathname_copy
   &pathname_installation);
@@ -54,10 +55,10 @@ our @EXPORT = qw( &pathname_find &pathname_findall &pathname_kpsewhich
 # Some indicators that this is not sufficient? (calls to libraries/externals???)
 # PRELIMINARY test, probably need to be even more careful
 my $ISWINDOWS   = $^O =~ /^(MSWin|NetWare|cygwin)/i;
-my $SEP         = ($ISWINDOWS ? '\\' : '/');           # [CONSTANT]
-my $KPATHSEP    = ($ISWINDOWS ? ';' : ':');            # [CONSTANT]
-my $LITERAL_RE  = '(?:literal)(?=:)';                  # [CONSTANT]
-my $PROTOCOL_RE = '(?:https|http|ftp)(?=:)';           # [CONSTANT]
+my $SEP         = ($ISWINDOWS ? '\\' : '/');    # [CONSTANT]
+my $KPATHSEP    = ($ISWINDOWS ? ';'  : ':');    # [CONSTANT]
+my $LITERAL_RE  = '(?:literal)(?=:)';           # [CONSTANT]
+my $PROTOCOL_RE = '(?:https|http|ftp)(?=:)';    # [CONSTANT]
 
 #======================================================================
 # pathname_make(dir=>dir, name=>name, type=>type);
@@ -72,8 +73,8 @@ sub pathname_make {
     foreach my $d (@dirs) {
       $pathname =~ s|\Q$SEP\E$||; $dir =~ s|^\Q$SEP\E||;
       $pathname .= $SEP . $dir; } }
-  $pathname .= $SEP if $pathname && $pieces{name} && $pathname !~ m|\Q$SEP\E$|;
-  $pathname .= $pieces{name} if $pieces{name};
+  $pathname .= $SEP                if $pathname && $pieces{name} && $pathname !~ m|\Q$SEP\E$|;
+  $pathname .= $pieces{name}       if $pieces{name};
   $pathname .= '.' . $pieces{type} if $pieces{type};
   return pathname_canonical($pathname); }
 
@@ -81,6 +82,7 @@ sub pathname_make {
 # If pathname is absolute, dir starts with volume or '/'
 sub pathname_split {
   my ($pathname) = @_;
+  return unless defined $pathname;
   $pathname = pathname_canonical($pathname);
   my ($vol, $dir, $name) = File::Spec->splitpath($pathname);
   # Hmm, for /, we get $dir = / but we want $vol='/'  ?????
@@ -98,6 +100,7 @@ use Carp;
 # AND, care about symbolic links and collapsing ../ !!!
 sub pathname_canonical {
   my ($pathname) = @_;
+  return unless defined $pathname;
   if ($pathname =~ /^($LITERAL_RE)/) {
     return $pathname; }
   # Don't call pathname_is_absolute, etc, here, cause THEY call US!
@@ -116,7 +119,8 @@ sub pathname_canonical {
   $pathname =~ s|/\./|/|g;
   # Collapse any foo/.. patterns, but not ../..
   while ($pathname =~ s|/(?!\.\./)[^/]+/\.\.(/\|$)|$1|) { }
-  $pathname =~ s|^\./||;
+  # Reduce ./foo to foo, but preserve ./
+  $pathname =~ s|^\./(.)|$1|;
   return (defined $urlprefix ? $urlprefix . $pathname : $pathname); }
 
 # Convenient extractors;
@@ -138,6 +142,7 @@ sub pathname_type {
 # Note that this returns ONLY recognized protocols!
 sub pathname_protocol {
   my ($pathname) = @_;
+  return unless defined $pathname;
   return ($pathname =~ /^($PROTOCOL_RE|$LITERAL_RE)/ ? $1 : 'file'); }
 
 #======================================================================
@@ -173,6 +178,20 @@ sub pathname_is_contained {
   # If the relative pathname starts with "../" that it apparently is NOT underneath base!
   return ($rel =~ m|^\.\.(?:/\|\Q$SEP\E)| ? undef : $rel); }
 
+# Check whether a pathname is a raw TeX source or definition
+sub pathname_is_raw {
+  my ($pathname) = @_;
+  return ($pathname =~ /\.(tex|pool|sty|cls|clo|cnf|cfg|ldf|def|dfu)$/); }
+
+# Check whether a pathname is reloadable as a TeX definition
+sub pathname_is_reloadable {
+  my ($pathname) = @_;
+  my ($dir, $name, $type) = pathname_split($pathname);
+  # babel.sty exception:
+  # we know the same .ldf file may be reloaded with a different option,
+  # to load an adjacently defined language, so allow that.
+  return $type eq 'ldf'; }
+
 # pathname_relative($pathname,$base) => $relativepathname
 # If $pathname is an absolute, non-URL pathname,
 # return the pathname relative to $base,
@@ -181,6 +200,7 @@ sub pathname_is_contained {
 # we _could_ make relative...
 sub pathname_relative {
   my ($pathname, $base) = @_;
+  return unless defined $pathname;
   $pathname = pathname_canonical($pathname);
   return ($base && pathname_is_absolute($pathname) && !pathname_is_url($pathname)
     ? File::Spec->abs2rel($pathname, pathname_canonical($base))
@@ -188,28 +208,43 @@ sub pathname_relative {
 
 sub pathname_absolute {
   my ($pathname, $base) = @_;
+  return unless defined $pathname;
   $pathname = pathname_canonical($pathname);
   return (!pathname_is_absolute($pathname) && !pathname_is_url($pathname)
     ? File::Spec->rel2abs($pathname, ($base ? pathname_canonical($base) : pathname_cwd()))
     : $pathname); }
 
+sub pathname_to_url {
+  my ($pathname) = @_;
+  return unless defined $pathname;
+  my $relative_pathname = pathname_relative($pathname);
+  if ($SEP ne '/') {
+    $relative_pathname = join('/', split(/\Q$SEP\E/, $relative_pathname)); }
+  return $relative_pathname; }
+
 #======================================================================
 # Actual file system operations.
 sub pathname_timestamp {
   my ($pathname) = @_;
   return -f $pathname ? (stat($pathname))[9] : 0; }
 
+our $CWD = undef;
+# DO NOT use pathname_cwd, unless you also use pathname_chdir to change dirs!!!
 sub pathname_cwd {
-  if (my $cwd = cwd()) {
-    return pathname_canonical($cwd); }
-  else {
-    # Fatal not imported
-    die "INTERNAL: Could not determine current working directory (cwd)"
-      . "Perhaps a problem with Perl's locale settings?"; } }
+  if (!defined $CWD) {
+    if (my $cwd = cwd()) {
+      $CWD = pathname_canonical($cwd); }
+    else {
+      # Fatal not imported
+      die "INTERNAL: Could not determine current working directory (cwd)"
+        . "Perhaps a problem with Perl's locale settings?"; } }
+  return $CWD; }
 
 sub pathname_chdir {
   my ($directory) = @_;
-  return chdir($directory); }
+  chdir($directory);
+  pathname_cwd();    # RE-cache $CWD!
+  return; }
 
 sub pathname_mkdir {
   my ($directory) = @_;
@@ -272,18 +307,17 @@ sub pathname_installation {
 sub pathname_find {
   my ($pathname, %options) = @_;
   return unless $pathname;
+  $options{findfirst} = 1;
   my @paths = candidate_pathnames($pathname, %options);
-  foreach my $path (@paths) {
-    return $path if -f $path; }
-  return; }
+  return $paths[0]; }
 
 sub pathname_findall {
   my ($pathname, %options) = @_;
   return unless $pathname;
-  my @paths = candidate_pathnames($pathname, %options);
-  return grep { -f $_ } @paths; }
+  $options{findfirst} = undef;
+  return candidate_pathnames($pathname, %options); }
 
-# It's presumably cheep to concatinate all the pathnames,
+# It's presumably cheap to concatinate all the pathnames,
 # relative to the cost of testing for files,
 # and this simplifies overall.
 sub candidate_pathnames {
@@ -304,7 +338,7 @@ sub candidate_pathnames {
           $pathdir);
         push(@dirs, $pp) unless grep { $pp eq $_ } @dirs; } }    # but only include each dir ONCE
     push(@dirs, pathname_concat($cwd, $pathdir)) unless @dirs;    # At least have the current directory!
-           # And, if installation dir specified, append it.
+        # And, if installation dir specified, append it.
     if (my $subdir = $options{installation_subdir}) {
       push(@dirs, map { pathname_concat($_, $subdir) } @INSTALLDIRS); } }
 
@@ -314,69 +348,92 @@ sub candidate_pathnames {
     push(@exts, '.' . $options{type}); }
   if ($options{types}) {
     foreach my $ext (@{ $options{types} }) {
-      if ($ext eq '') { push(@exts, ''); }
+      if    ($ext eq '') { push(@exts, ''); }
       elsif ($ext eq '*') {
         push(@exts, '.*', ''); }
-      elsif ($pathname =~ /\.\Q$ext\E$/i) {
-        push(@exts, ''); }
       else {
+        if ($pathname =~ /\.\Q$ext\E$/i) {
+          push(@exts, ''); }
         push(@exts, '.' . $ext); } } }
   push(@exts, '') unless @exts;
 
-  my @paths = ();
+  my @paths        = ();
+  my @nocase_paths = ();
+  # Precompute the intended regexes to apply
+  my @regexes = ();
+  foreach my $ext (@exts) {
+    if ($name eq '*') {    # Unfortunately, we've got to test the file system NOW...
+      if ($ext eq '.*') {    # everything, except hidden files
+        push(@regexes, [qr/^[^.]/, qr/^[^.]/]); }
+      else {
+        push(@regexes, [qr/\Q$ext\E$/i,
+            qr/\Q$ext\E$/]); } }
+    elsif ($ext eq '.*') {
+      push(@regexes, [qr/^\Q$name\E\.\w+$/i,
+          qr/^\Q$name\E\.\w+$/]); }
+    else {
+      push(@regexes, [qr/^\Q$name\E\Q$ext\E$/i,
+          qr/^\Q$name\E\Q$ext\E$/]); } }
   # Now, combine; precedence to leading directories.
   foreach my $dir (@dirs) {
-    foreach my $ext (@exts) {
-      if ($name eq '*') {    # Unfortunately, we've got to test the file system NOW...
-        if ($ext eq '.*') {    # everything
-          opendir(DIR, $dir) or next;
-          push(@paths, map { pathname_concat($dir, $_) } grep { !/^\./ } readdir(DIR));
-          closedir(DIR); }
-        else {
-          opendir(DIR, $dir) or next;    # ???
-          push(@paths, map { pathname_concat($dir, $_) } grep { /\Q$ext\E$/ } readdir(DIR));
-          closedir(DIR); } }
-      elsif ($ext eq '.*') {             # Unfortunately, we've got to test the file system NOW...
-        opendir(DIR, $dir) or next;      # ???
-        push(@paths, map { pathname_concat($dir, $_) } grep { /^\Q$name\E\.\w+$/ } readdir(DIR));
-        closedir(DIR); }
-      else {
-        push(@paths, pathname_concat($dir, $name . $ext)); } } }
-  return @paths; }
+    opendir(DIR, $dir) or next;
+    my @dir_files = readdir(DIR);
+    closedir(DIR);
+    for my $local_file (@dir_files) {
+      for my $regex_pair (@regexes) {
+        my ($i_regex, $regex) = @$regex_pair;
+        if ($local_file =~ m/$i_regex/) {
+          my $full_file = pathname_concat($dir, $local_file);
+          push(@nocase_paths, $full_file);
+          if ($local_file =~ m/$regex/) {
+            # if we are only interested in the first match, return it:
+            return ($full_file) if $options{findfirst};
+            push(@paths, $full_file); } } } } }
+  # Fallback: if no strict matches were found, return any existing case-insensitive matches
+  # Defer the -f check until we are sure we need it, to keep the usual cases fast.
+  return @paths ? @paths : @nocase_paths; }
 
 #======================================================================
-our $kpsewhich = which($ENV{LATEXML_KPSEWHICH} || 'kpsewhich');
-our $kpse_cache = undef;
+our $kpsewhich      = which($ENV{LATEXML_KPSEWHICH} || 'kpsewhich');
+our $kpse_cache     = undef;
+our $kpse_toolchain = "";
 
 sub pathname_kpsewhich {
   my (@candidates) = @_;
-  return unless $kpsewhich;
+  return             unless $kpsewhich && @candidates;
   build_kpse_cache() unless $kpse_cache;
   foreach my $file (@candidates) {
     if (my $result = $$kpse_cache{$file}) {
       return $result; } }
   # If we've failed to read the cache, try directly calling kpsewhich
   # For multiple calls, this is slower in general. But MiKTeX, eg., doesn't use texmf ls-R files!
-  my $files = join(' ', @candidates);
-  if ($kpsewhich && (my $result = `"$kpsewhich" $files`)) {
-    if ($result =~ /^\s*(.+?)\s*\n/s) {
-      return $1; } }
+  if ($kpse_toolchain) {
+    push(@candidates, $kpse_toolchain); }
+  if ($kpsewhich && open(my $resfh, '-|', $kpsewhich, @candidates)) {
+    my $result = <$resfh>;     # we only need the first line
+    { local $/; <$resfh>; }    # discard the rest of the output
+    close($resfh);             # ignore exit status (only one of @candidates exists, usually)
+    if ($result) {
+      chomp $result;
+      return $result; } }
   return; }
 
 sub build_kpse_cache {
   $kpse_cache = {};    # At least we've tried.
   return unless $kpsewhich;
-       # This finds ALL the directories looked for for any purposes, including docs, fonts, etc
-  my $texmf = `"$kpsewhich" --expand-var \'\\\$TEXMF\'`; chomp($texmf);
+  # This finds ALL the directories looked for for any purposes, including docs, fonts, etc
+  $kpse_toolchain = "--miktex-admin" if ($ENV{"LATEXML_KPSEWHICH_MIKTEX_ADMIN"});
+  my $texmf = `"$kpsewhich" --expand-var \'\\\$TEXMF\' $kpse_toolchain`; chomp($texmf);
   # These are directories which contain the tex related files we're interested in.
   # (but they're typically below where the ls-R indexes are!)
-  my $texpaths = `"$kpsewhich" --show-path tex`; chomp($texpaths);
-  my @filters = ();
+  my $texpaths = `"$kpsewhich" --show-path tex $kpse_toolchain`; chomp($texpaths);
+  my @filters  = ();
   foreach my $path (split(/$KPATHSEP/, $texpaths)) {
     $path =~ s/^!!//; $path =~ s|//+$|/|;
     push(@filters, $path) if -d $path; }
   $texmf =~ s/^["']//; $texmf =~ s/["']$//;
-  $texmf =~ s/^\s*\\\{(.+?)}\s*/$1/s;
+  $texmf =~ s/^\s*\\\{(.+?)}\s*$/$1/s;
+  $texmf =~ s/\{\}//g;
   my @dirs = split(/,/, $texmf);
   foreach my $dir (@dirs) {
     $dir =~ s/^!!//;
@@ -389,11 +446,11 @@ sub build_kpse_cache {
       while (<$LSR>) {
         chop;
         next unless $_;
-        if (/^%/) { }
+        if    (/^%/) { }
         elsif (/^(.*?):$/) {    # Move to a new subdirectory
           $subdir = $1;
-          $subdir =~ s|^\./||;    # remove prefix
-          my $d = $dir . '/' . $subdir;    # Hopefully OS safe, for comparison?
+          $subdir =~ s|^\./||;                             # remove prefix
+          my $d = $dir . '/' . $subdir;                    # Hopefully OS safe, for comparison?
           $skip = !grep { $d =~ /^\Q$_\E/ } @filters; }    # check if one of the TeX paths
         elsif (!$skip) {
           # Is it safe to use '/' here?
@@ -407,7 +464,7 @@ sub build_kpse_cache {
 
 __END__
 
-=pod 
+=pod
 
 =head1 NAME
 
@@ -464,6 +521,20 @@ Returns whether the pathname C<$path> appears to be an absolute pathname.
 
 Returns whether the pathname C<$path> appears to be a url, rather than local file.
 
+=item C<< $boole = pathname_is_literaldata($path); >>
+
+Returns whether the pathname C<$path> is actually a blob of literal data,
+with a leading "literal:" protocol.
+
+=item C<< $boole = pathname_is_raw($path); >>
+
+Check if pathname indicates a raw TeX source or definition file.
+
+=item C<< $boole = pathname_is_reloadable($path); >>
+
+Check for pathname exceptions where the same TeX definition file
+can be meaningfully reloaded. For example, babel.sty ".ldf" files
+
 =item C<< $rel = pathname_is_contained($path,$base); >>
 
 Checks whether C<$path> is underneath the directory C<$base>; if so
@@ -481,6 +552,11 @@ Returns the absolute pathname resulting from interpretting
 C<$path> relative to the directory C<$base>.  If C<$path>
 is already absolute, it is returned unchanged.
 
+=item C<< $relative_url = pathname_to_url($path); >>
+
+Creates a local, relative URL for a given pathname,
+also ensuring proper path separators on non-Unix systems.
+
 =back
 
 =head2 File System Operations
@@ -509,9 +585,9 @@ It preserves the timestamp of C<$source>.
 
 =item C<< $path = pathname_find($name,%options); >>
 
-Finds the first file named C<$name> that exists 
+Finds the first file named C<$name> that exists
 and that matches the specification
-in the keywords C<%options>.  
+in the keywords C<%options>.
 An absolute pathname is returned.
 
 If C<$name> is not already an absolute pathname, then
@@ -534,6 +610,13 @@ The type C<*> matches any extension.
 Like C<pathname_find>,
 but returns I<all> matching (absolute) paths that exist.
 
+=item C<< $path = pathname_kpsewhich(@names); >>
+
+Attempt to find a candidate name via the external C<kpsewhich>
+capability of the system's TeX toolchain. If C<kpsewhich> is
+not available, or the file is not found, returns a
+Perl undefined value.
+
 =back
 
 =head1 AUTHOR
@@ -546,4 +629,3 @@ Public domain software, produced as part of work done by the
 United States Government & not subject to copyright in the US.
 
 =cut
-
